﻿-----------------------------------------------------------------------------
--
-- data
--   \aura
--     \<playerName>
--       - count
--       \auraList
--         \<auraName>
--		     - count, isBuff
--           \times
--
-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AssessmentAura")
L:RegisterTranslations("enUS", function() return {
	["Aura Recipient List"] = true,
	["Auras"] = true,
	["Aura Details"] = true,
	["Aura Times"] = true,
	["Buff/Debuff"] = true,

	["Aura"] = true,
	["Aura Options."] = true,
	["Aura Times"] = true,
	["Gather information on when auras are applied."] = true,
} end)

L:RegisterTranslations("zhCN", function() return {
	["Aura Recipient List"] = "Buff&Debuff的接受者列表",
	["Auras"] = "Buffs&Debuffs",
	["Aura Details"] = "Buff&Debuff细节",
	["Aura Times"] = "Buff&Debuff时间",
	["Buff/Debuff"] = "Buff/Debuff",

	["Aura"] = "Buff&Debuff",
	["Aura Options."] = "Buff&Debuff的相关选项",
	["Aura Times"] = "Buff&Debuff时间",
	["Gather information on when auras are applied."] = "收集Buff&Debuff信息",
} end)

L:RegisterTranslations("zhCN", function() return {
	["Aura Recipient List"] = "Buff&Debuff的接受者列表",
	["Auras"] = "Buffs&Debuffs",
	["Aura Details"] = "Buff&Debuff細節",
	["Aura Times"] = "Buff&Debuff時間",
	["Buff/Debuff"] = "Buff/Debuff",

	["Aura"] = "Buff&Debuff",
	["Aura Options."] = "Buff&Debuff的相關選項",
	["Aura Times"] = "Buff&Debuff時間",
	["Gather information on when auras are applied."] = "收集Buff&Debuff信息",
} end)


L:RegisterTranslations("koKR", function() return {
	["Aura Recipient List"] = "효과 수령인 목록",
	["Auras"] = "효과",
	["Aura Details"] = "효과 상세 정보",
	["Aura Times"] = "효과 시간",
	["Buff/Debuff"] = "버프/디버프",

	["Aura"] = "효과",
	["Aura Options."] = "효과 설정입니다.",
	["Aura Times"] = "효과 시간",
	["Gather information on when auras are applied."] = "받은 효과의 정보를 수집합니다.",
} end)

L:RegisterTranslations("frFR", function() return {
	["Aura Recipient List"] = "Aura Recipient List",
	["Auras"] = "Auras",
	["Aura Details"] = "D\195\169tails des Auras",
	["Aura Times"] = "Temps des Aura",
	["Buff/Debuff"] = "Buff\195\169buff",

	["Aura"] = "Aura",
	["Aura Options."] = "Options des Aura.",
	["Aura Times"] = "Temps des Auras",
	["Gather information on when auras are applied."] = "Gather information on when auras are applied.",
} end)

-----------------------------------------------------------------------------

local AssessmentAura = Assessment:NewModule("Aura", "AceEvent-2.0", "Parser-3.0")
AssessmentAura.db = Assessment:AcquireDBNamespace("Aura")
Assessment:RegisterDefaults("Aura", "profile", {
	enable				= true,
	gatherTimeFilter	= "NONE",
	buffColor			= { 0.3, 0.8, 0.3 },
	debuffColor			= { 0.8, 0.3, 0.3 },
})

-----------------------------------------------------------------------------

local profile
local gatherTimeDataMethod

local emptyMethod = Assessment.EmptyMethod
local new = Assessment.NewTable
local pairs = pairs

-----------------------------------------------------------------------------

Assessment.TagData.AuraCount =
{
	method = function(data)
				local fightData = data.fightData
				if not fightData then return 0 end
				
				local aura = fightData.aura
				if not aura then return 0 end
				
				local playerAura = aura[data.key]
				if not playerAura then return 0 end

				return playerAura.count
			 end,
	data = "aura",
	guaranteeResult = true,
	numeric = true,
}

-----------------------------------------------------------------------------

Assessment.ListType.AuraRecipients =
{
	method = function(fightData)
				if not fightData.aura then return emptyMethod end
				return pairs(fightData.aura)
			 end,
	data = "aura",
	description = L["Aura Recipient List"],
}

-----------------------------------------------------------------------------

Assessment.ColorType.BuffDebuff = 
{
	method = function(fightData, data)
				return data.value.isBuff and profile.buffColor or profile.debuffColor
			 end,
	description = L["Buff/Debuff"]
}

-----------------------------------------------------------------------------

Assessment.DefaultDisplaySets.AuraTimes =
{
	titleTag			= ("%s: [WindowParameter(1)]"):format(L["Aura Times"]),
	listType			= "EnumerateTable",
	listDataTag			= "aura,[WindowParameter(1)],auraList,[WindowParameter(2)],times",
	barValueTag			= "0",
	filterTag			= "1",
	fightIndex			= -1,
	text				= { count = 2, { tag = "[WindowParameter(2)] [Rank]" }, { tag = "[Value:FormatTime]" } },
	description			= L["Aura Times"],
	selectable			= false,
	favorite			= false,
	defaultColor		= { 0.8, 0.8, 0.5 },
}

Assessment.DefaultDisplaySets.AuraDetails =
{
	titleTag			= ("%s: [WindowParameter(1)]"):format(L["Aura Details"]),
	listType			= "EnumerateTable",
	listDataTag			= "aura,[WindowParameter(1)],auraList",
	barValueTag			= "Count",
	filterTag			= "1",
	fightIndex			= -1,
	text				= { count = 2, { tag = "[Rank]. [Key]" }, { tag = "[BarValue]" } },
	description			= L["Aura Details"],
	selectable			= false,
	colorTypeList		= { "BuffDebuff", count = 1 },
	defaultColor		= { 0.8, 0.8, 0.5 },
	favorite			= false,
	onClickType			= "SHOW_DISPLAY_SET",
	onClickDisplaySet	= L["Aura Times"],
	onClickData			= "[WindowParameter(1)],[Key]",
}

Assessment.DefaultDisplaySets.Auras =
{
	titleTag			= L["Auras"],
	listType			= "GroupMembers", 
	barValueTag			= "AuraCount", 
	filterTag			= "BarValue ~= 0",
	fightIndex			= 1,
	description			= L["Auras"],
	defaultColor		= { 0.8, 0.8, 0.5 },
	colorTypeList		= { "Class", count = 1 },
	onClickType			= "SHOW_DISPLAY_SET",
	onClickDisplaySet	= L["Aura Details"],
	onClickData			= "[Key]",
	favorite			= false,
}

-----------------------------------------------------------------------------

Assessment.ListType.EnumerateTable.updateData.aura = "aura"

-----------------------------------------------------------------------------

Assessment.Options.args.data.args.aura =
{
	type = "group",
	name = L["Aura"],
	desc = L["Aura Options."],
	args =
	{
		gatherTimeData =
		{
			type = "text",
			name = L["Aura Times"],
			desc = L["Gather information on when auras are applied."],
			get = function() return profile.gatherTimeFilter end,
			set = function(v) profile.gatherTimeFilter = v AssessmentAura:OnInitialize() end,
			validate = Assessment.GatherFilterValidate,
		},
	}
}

-----------------------------------------------------------------------------

function AssessmentAura:OnInitialize()
	profile = self.db.profile
	gatherTimeDataMethod = Assessment.GatherFilterList[profile.gatherTimeFilter].method
end

function AssessmentAura:OnEnable()
	self:RegisterParserEvent({ eventType = "Aura" }, "OnParserAuraEvent")
end

function AssessmentAura:OnDisable()
end

-----------------------------------------------------------------------------

function AssessmentAura:OnParserAuraEvent(eventData)
	local auraTime = time()
	local time = GetTime()

	for data in self:IterateUpdateDataTables() do
		data.dataUpdateTimes.aura = time
		local aura = data.aura
		if not aura then
			aura = new()
			data.aura = aura
		end

		local recipientName = eventData.recipientName
		local playerAura = aura[recipientName]
		local auraList
		if playerAura then
			playerAura.count = playerAura.count + 1
			auraList = playerAura.auraList
		else
			playerAura = new()
			playerAura.count = 1

			auraList = new()
			playerAura.auraList = auraList

			aura[recipientName] = playerAura
		end

		local abilityName = eventData.abilityName
		local auraData = auraList[abilityName]
		if auraData then
			auraData.count = auraData.count + 1
		else
			auraData = new()
			auraData.count = 1
			auraData.isBuff = eventData.isBuff
			auraList[abilityName] = auraData
		end
		
		if gatherTimeDataMethod(recipientName) then
			local times = auraData.times
			if not times then
				times = new()
				auraData.times = times
			end
			times[#times+1] = auraTime
		end
	end
end

-----------------------------------------------------------------------------
